#ifndef nsNetCID_h__
#define nsNetCID_h__

//
// nsStreamObserverProxy
//
#define NS_STREAMOBSERVERPROXY_CLASSNAME \
    "Stream Observer Proxy"
#define NS_STREAMOBSERVERPROXY_CONTRACTID \
    "@mozilla.org/network/stream-observer-proxy;1"
#define NS_STREAMOBSERVERPROXY_CID                   \
{ /* 51fa28c7-74c0-4b85-9c46-d03faa7b696b */         \
    0x51fa28c7,                                      \
    0x74c0,                                          \
    0x4b85,                                          \
    {0x9c, 0x46, 0xd0, 0x3f, 0xaa, 0x7b, 0x69, 0x6b} \
}

//
// nsStreamListenerProxy
//
#define NS_STREAMLISTENERPROXY_CLASSNAME \
    "Stream Listener Proxy"
#define NS_STREAMLISTENERPROXY_CONTRACTID \
    "@mozilla.org/network/stream-listener-proxy;1"
#define NS_STREAMLISTENERPROXY_CID                   \
{ /* 96c48f15-aa8a-4da7-a9d5-e842bd76f015 */         \
    0x96c48f15,                                      \
    0xaa8a,                                          \
    0x4da7,                                          \
    {0xa9, 0xd5, 0xe8, 0x42, 0xbd, 0x76, 0xf0, 0x15} \
}

//
// nsStreamProviderProxy
//
#define NS_STREAMPROVIDERPROXY_CLASSNAME \
    "Stream Provider Proxy"
#define NS_STREAMPROVIDERPROXY_CONTRACTID \
    "@mozilla.org/network/stream-provider-proxy;1"
#define NS_STREAMPROVIDERPROXY_CID                   \
{ /* ae964fcf-9c27-40f7-9bbd-78894bfc1f31 */         \
    0xae964fcf,                                      \
    0x9c27,                                          \
    0x40f7,                                          \
    {0x9b, 0xbd, 0x78, 0x89, 0x4b, 0xfc, 0x1f, 0x31} \
}

//
// nsSimpleStreamListener
//
#define NS_SIMPLESTREAMLISTENER_CLASSNAME \
    "Simple Stream Listener"
#define NS_SIMPLESTREAMLISTENER_CONTRACTID \
    "@mozilla.org/network/simple-stream-listener;1"
#define NS_SIMPLESTREAMLISTENER_CID                  \
{ /* fb8cbf4e-4701-4ba1-b1d6-5388e041fb67 */         \
    0xfb8cbf4e,                                      \
    0x4701,                                          \
    0x4ba1,                                          \
    {0xb1, 0xd6, 0x53, 0x88, 0xe0, 0x41, 0xfb, 0x67} \
}

//
// nsSimpleStreamProvider
//
#define NS_SIMPLESTREAMPROVIDER_CLASSNAME \
    "Simple Stream Provider"
#define NS_SIMPLESTREAMPROVIDER_CONTRACTID \
    "@mozilla.org/network/simple-stream-provider;1"
#define NS_SIMPLESTREAMPROVIDER_CID                  \
{ /* f9f6a519-4efb-4f36-af40-2a5ec3992710 */         \
    0xf9f6a519,                                      \
    0x4efb,                                          \
    0x4f36,                                          \
    {0xaf, 0x40, 0x2a, 0x5e, 0xc3, 0x99, 0x27, 0x10} \
}

//
// nsAsyncStreamObserver
//
#define NS_ASYNCSTREAMOBSERVER_CID                   \
{ /* fcc7c380-91b3-11d3-8cd9-0060b0fc14a3 */         \
    0xfcc7c380,                                      \
    0x91b3,                                          \
    0x11d3,                                          \
    {0x8c, 0xd9, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3} \
}

//
// nsAsyncStreamListener
//
#define NS_ASYNCSTREAMLISTENER_CID                   \
{ /* 60047bb2-91c0-11d3-8cd9-0060b0fc14a3 */         \
    0x60047bb2,                                      \
    0x91c0,                                          \
    0x11d3,                                          \
    {0x8c, 0xd9, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3} \
}


#endif /* !nsNetCID_h__ */
