/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "nsIDOMHTMLElement.idl"

[scriptable, uuid(a6cf90b2-15b3-11d2-932e-00805f8add32)]
interface nsIDOMHTMLTableElement : nsIDOMHTMLElement
{
           attribute nsIDOMHTMLTableCaptionElement caption;
           attribute nsIDOMHTMLTableSectionElement tHead;
           attribute nsIDOMHTMLTableSectionElement tFoot;
  readonly attribute nsIDOMHTMLCollection          rows;
  readonly attribute nsIDOMHTMLCollection          tBodies;
           attribute DOMString                     align;
           attribute DOMString                     bgColor;
           attribute DOMString                     border;
           attribute DOMString                     cellPadding;
           attribute DOMString                     cellSpacing;
           attribute DOMString                     frame;
           attribute DOMString                     rules;
           attribute DOMString                     summary;
           attribute DOMString                     width;
  nsIDOMHTMLElement         createTHead();
  void                      deleteTHead();
  nsIDOMHTMLElement         createTFoot();
  void                      deleteTFoot();
  nsIDOMHTMLElement         createCaption();
  void                      deleteCaption();
  nsIDOMHTMLElement         insertRow(in long index);
  void                      deleteRow(in long index);
};
