/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 2000 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */
 
#include "nsISupports.idl"

%{C++
class nsIEditor;
class nsCString;
%}

[scriptable, uuid(D937F1C2-6FAB-11d4-B37D-004095E27A10)]
interface nsIEditorController : nsISupports
{

  /** init the controller
   *  param: commandRefCon = a cookie that is passed to commands
   */
  void Init( in nsISupports commandRefCon );

  /** Set the cookie that is passed to commands
   */
  void SetCommandRefCon( in nsISupports commandRefCon );

  /** Execute the command identified by the ID string
   *  param: commandName  ID string of the command
   */
  void DoCommand( in wstring commandName );

  /* Should we expose nsEditorController::IsCommandEnabled() as well? */
};
