/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "nsIDOMDocument.idl"

[scriptable, uuid(a6cf9084-15b3-11d2-932e-00805f8add32)]
interface nsIDOMHTMLDocument : nsIDOMDocument
{
           attribute DOMString            title;
  readonly attribute DOMString            referrer;
           attribute DOMString            domain;
  readonly attribute DOMString            URL;
           attribute nsIDOMHTMLElement    body;
  readonly attribute nsIDOMHTMLCollection images;
  readonly attribute nsIDOMHTMLCollection applets;
  readonly attribute nsIDOMHTMLCollection links;
  readonly attribute nsIDOMHTMLCollection forms;
  readonly attribute nsIDOMHTMLCollection anchors;
           attribute DOMString            cookie;

  [noscript] void           open();
  void                      close();
  [noscript] void           write(in DOMString text);
  [noscript] void           writeln(in DOMString text);
  nsIDOMNodeList            getElementsByName(in DOMString elementName);
};
