//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by MfcEmbed.rc
//
#define IDD_ABOUTBOX                    100
#define IDR_MAINFRAME                   128
#define IDR_MOZEMBTYPE                  129
#define IDR_CTXMENU_DOCUMENT            130
#define IDR_CTXMENU_LINK                131
#define IDR_CTXMENU_TEXT                132
#define IDR_CTXMENU_IMAGE               133
#define IDD_PROMPT_DIALOG               134
#define IDD_PROMPT_PASSWORD_DIALOG      135
#define	IDD_PROMPT_USERPASS_DIALOG		136
#define	IDD_PROFILES					137
#define	IDD_PROFILE_NEW					138
#define	IDD_PROFILE_RENAME				139
#define ID_URL_BAR                      1001
#define ID_PROG_BAR                     1002
#define IDC_PROMPT_ANSWER               1003
#define IDC_PROMPT_QUESTION             1004
#define IDC_PROMPT_TEXT                 1005
#define IDC_USERNAME                    1006
#define IDC_PASSWORD                    1007
#define IDC_CHECK_SAVE_PASSWORD         1008
#define IDC_USERNAME_LABEL              1009
#define IDC_PASSWORD_LABEL              1010
#define IDC_LIST1						1011
#define IDC_PROF_RENAME					1012
#define IDC_PROF_DELETE					1013
#define IDC_PROF_NEW					1014
#define IDC_CHECK_ASK_AT_START			1015
#define IDC_NEW_PROF_NAME				1016
#define IDC_LOCALE_COMBO				1017
#define IDC_NEW_NAME					1018
#define ID_NAV_BACK                     32773
#define ID_NAV_FORWARD                  32774
#define ID_NAV_HOME                     32775
#define ID_NAV_STOP                     32776
#define ID_NAV_RELOAD                   32777
#define ID_EDIT_SELECT_NONE             32778
#define ID_NEW_BROWSER                  32779
#define ID_VIEW_SOURCE                  32780
#define ID_VIEW_INFO                    32781
#define ID_VIEW_IMAGE                   32782
#define ID_OPEN_LINK_IN_NEW_WINDOW      32783
#define ID_SAVE_LINK_AS                 32784
#define ID_SAVE_IMAGE_AS                32785
#define ID_COPY_LINK_LOCATION           32786
#define	ID_MANAGE_PROFILES				32787

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_3D_CONTROLS					1
#define _APS_NEXT_RESOURCE_VALUE        140
#define _APS_NEXT_COMMAND_VALUE         32788
#define _APS_NEXT_CONTROL_VALUE         1019
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
