/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "nsIDOMEvent.idl"

[scriptable, uuid(8e440d86-886a-4e76-9e59-c13b939c9a4b)]
interface nsIDOMMutationEvent : nsIDOMEvent
{
  const unsigned short      MODIFICATION       = 1;
  const unsigned short      ADDITION           = 2;
  const unsigned short      REMOVAL            = 3;

  readonly attribute nsIDOMNode       relatedNode;
  readonly attribute DOMString        prevValue;
  readonly attribute DOMString        newValue;
  readonly attribute DOMString        attrName;
  readonly attribute unsigned short   attrChange;
  void                      initMutationEvent(in DOMString typeArg,
                                              in boolean canBubbleArg,
                                              in boolean cancelableArg,
                                              in nsIDOMNode relatedNodeArg,
                                              in DOMString prevValueArg,
                                              in DOMString newValueArg,
                                              in DOMString attrNameArg);
};
