/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "domstubs.idl"

[scriptable, uuid(a6cf90f2-15b3-11d2-932e-00805f8add32)]
interface nsIDOMNSRange : nsISupports
{
  nsIDOMDocumentFragment    createContextualFragment(in DOMString fragment);
  boolean                   isValidFragment(in DOMString fragment);

  // Is the point contained in the range?
  boolean                   isPointInRange(in nsIDOMNode parent,
                                           in long offset);

  // comparePoint returns
  //   -1 if point is before range,
  //    0 if point is in range,
  //    1 if point is after range
  // Sort of a strcmp for ranges.
  short                     comparePoint(in nsIDOMNode parent, in long offset);

  // Does the node intersect the range?
  boolean                   intersectsNode(in nsIDOMNode n);

  // HOW does the node intersect the range?  Four possible values:
  const unsigned short NODE_BEFORE = 0;
  const unsigned short NODE_AFTER = 1;
  const unsigned short NODE_BEFORE_AND_AFTER = 2;
  const unsigned short NODE_INSIDE = 3;
  unsigned short            compareNode(in nsIDOMNode n);

  // now what the %&*!@ is this?
  void                      nSDetach();
};
