/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "domstubs.idl"

[scriptable, uuid(a6cf907b-15b3-11d2-932e-00805f8add32)]
interface nsIDOMNamedNodeMap : nsISupports
{
  nsIDOMNode                getNamedItem(in DOMString name);
  nsIDOMNode                setNamedItem(in nsIDOMNode arg)
                                  raises(DOMException);
  nsIDOMNode                removeNamedItem(in DOMString name)
                                  raises(DOMException);
  nsIDOMNode                item(in unsigned long index);
  readonly attribute  unsigned long        length;
  // Introduced in DOM Level 2:
  nsIDOMNode                getNamedItemNS(in DOMString namespaceURI, 
                                           in DOMString localName);
  // Introduced in DOM Level 2:
  nsIDOMNode                setNamedItemNS(in nsIDOMNode arg)
                                  raises(DOMException);
  // Introduced in DOM Level 2:
  nsIDOMNode                removeNamedItemNS(in DOMString namespaceURI, 
                                              in DOMString localName)
                                  raises(DOMException);
};
