/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Mozilla browser.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications, Inc.  Portions created by Netscape are
 * Copyright (C) 1999, Mozilla.  All Rights Reserved.
 * 
 * Contributor(s):
 */

#include "domstubs.idl"

[scriptable, uuid(a6cf90ce-15b3-11d2-932e-00805f8add32)]
interface nsIDOMRange : nsISupports
{

	const unsigned short START_TO_START = 0;
	const unsigned short START_TO_END = 1;
	const unsigned short END_TO_START = 2;
	const unsigned short END_TO_END = 3;

    readonly attribute nsIDOMNode startContainer;
    readonly attribute long startOffset;
    readonly attribute nsIDOMNode endContainer;
    readonly attribute long endOffset;

    readonly attribute boolean collapsed;
    readonly attribute nsIDOMNode commonAncestorContainer;

    void setStart(in nsIDOMNode parent, in long offset);
    void setStartBefore(in nsIDOMNode sibling);
    void setStartAfter(in nsIDOMNode sibling);
    void setEnd(in nsIDOMNode parent, in long offset);
    void setEndBefore(in nsIDOMNode sibling);
    void setEndAfter(in nsIDOMNode sibling);

    void collapse(in boolean toStart);

    void selectNode(in nsIDOMNode n);
    void selectNodeContents(in nsIDOMNode n);

    long  compareBoundaryPoints(in unsigned short how, in nsIDOMRange srcRange);

    void deleteContents();

    nsIDOMDocumentFragment extractContents();
    nsIDOMDocumentFragment cloneContents();

    void insertNode(in nsIDOMNode n);
    void surroundContents(in nsIDOMNode n);

    nsIDOMRange cloneRange();

    void detach();

    DOMString toString();
};

